@@ -17,7 +17,6 @@
 #include <readInputImage.h>
 #include "namespaceAlias.h"
 
-
 void
 makeLatLongMap (EnvmapImage &image,
                 IMF::Header &header,
@@ -27,7 +26,7 @@ makeLatLongMap (EnvmapImage &image,
                 int tileHeight,
                 IMF::LevelMode levelMode,
                 IMF::LevelRoundingMode roundingMode,
-                IMF::Compression compresssion,
+                IMF::Compression compression,
                 int mapWidth,
                 float filterRadius,
                 int numSamples,
