@@ -573,7 +573,6 @@ combine (vector <const char*> in,
     }
 
     inputs.clear();
-    fordelete.size();
 
     cout << "\n" << "Combine Success" << endl;
 }
@@ -596,7 +595,7 @@ separate (vector <const char*> in, const char* out, bool override)
     int numOutputs;
     vector<string> fornamecheck;
 
-    // add check for existance of the file
+    // add check for existence of the file
     try
     {
         MultiPartInputFile temp (filename.c_str());
@@ -662,7 +661,8 @@ separate (vector <const char*> in, const char* out, bool override)
     }
 
     delete inputimage;
-    cout << "\n" << "Seperate Success" << endl;
+    cout << "\n"
+         << "Separate Success" << endl;
 }
 
 void
