@@ -24,6 +24,9 @@
 #include <ImfTileDescription.h>
 #include "ImfNamespace.h"
 
+#include <codecvt>
+#include <locale>
+
 OPENEXR_IMF_INTERNAL_NAMESPACE_SOURCE_ENTER
 
 using IMATH_NAMESPACE::Box2i;
@@ -1893,5 +1896,11 @@ getChunkOffsetTableSize(const Header& header)
     
 }
 
+std::wstring
+WidenFilename (const char* filename)
+{
+    std::wstring_convert<std::codecvt_utf8<wchar_t>, wchar_t> converter;
+    return converter.from_bytes (filename);
+}
 
 OPENEXR_IMF_INTERNAL_NAMESPACE_SOURCE_EXIT
