@@ -14,12 +14,12 @@
 
 
 // GCC and Visual Studio SSE2 compiler flags
-#if defined __SSE2__ || (_MSC_VER >= 1300 && (_M_IX86 || _M_X64))
-    #define IMF_HAVE_SSE2 1
+#if defined __SSE2__ || (_MSC_VER && (_M_IX86 || _M_X64))
+#    define IMF_HAVE_SSE2 1
 #endif
 
-#if defined __SSE4_1__
-    #define IMF_HAVE_SSE4_1 1
+#if defined __SSE4_1__ || (_MSC_VER && (_M_IX86 || _M_X64))
+#    define IMF_HAVE_SSE4_1 1
 #endif
 
 // Compiler flags on e2k (MCST Elbrus 2000) architecture
@@ -43,8 +43,15 @@
     #define IMF_HAVE_F16C 1
 #endif
 
-extern "C"
-{
+#if defined(__ARM_NEON)
+#    define IMF_HAVE_NEON
+#endif
+
+#if defined(__aarch64__)
+#    define IMF_HAVE_NEON_AARCH64 1
+#endif
+
+extern "C" {
 #ifdef IMF_HAVE_SSE2
     #include <emmintrin.h>
     #include <mmintrin.h>
@@ -53,6 +60,25 @@ extern "C"
 #ifdef IMF_HAVE_SSE4_1
     #include <smmintrin.h>
 #endif
+
+#ifdef IMF_HAVE_NEON
+#    include <arm_neon.h>
+#endif
+
 }
 
+#include "OpenEXRConfigInternal.h"
+#ifdef OPENEXR_MISSING_ARM_VLD1
+/* Workaround for missing vld1q_f32_x2 in older gcc versions.  */
+
+__extension__ extern __inline float32x4x2_t
+    __attribute__ ((__always_inline__, __gnu_inline__, __artificial__))
+    vld1q_f32_x2 (const float32_t* __a)
+{
+    float32x4x2_t ret;
+    asm ("ld1 {%S0.4s - %T0.4s}, [%1]" : "=w"(ret) : "r"(__a) :);
+    return ret;
+}
+#endif
+
 #endif
