@@ -7,6 +7,7 @@
 
 #include "internal_coding.h"
 #include "internal_structs.h"
+#include "internal_util.h"
 #include "internal_xdr.h"
 
 #include <limits.h>
@@ -239,6 +240,7 @@ struct priv_chunk_leader
             int32_t level_y;
         };
     };
+    uint8_t _pad[4];
     union
     {
         int64_t deep_data[3];
@@ -290,7 +292,7 @@ extract_chunk_leader (
     rv = ctxt->do_read (
         ctxt,
         data,
-        ntoread * sizeof (int32_t),
+        (size_t)ntoread * sizeof (int32_t),
         &nextoffset,
         NULL,
         EXR_MUST_READ_ALL);
@@ -347,7 +349,7 @@ extract_chunk_leader (
             return ctxt->print_error (
                 ctxt,
                 EXR_ERR_BAD_CHUNK_LEADER,
-                "Invalid chunk size reconstructing chunk table: found out of range %ld",
+                "Invalid chunk size reconstructing chunk table: found out of range %"PRId64,
                 leaderdata->deep_data[1]);
         }
         leaderdata->packed_size = leaderdata->deep_packed_size;
@@ -408,8 +410,21 @@ read_and_validate_chunk_leader (
     if (part->storage_mode == EXR_STORAGE_SCANLINE ||
         part->storage_mode == EXR_STORAGE_DEEP_SCANLINE)
     {
-        *indexio = (leader.scanline_y - part->data_window.min.y) /
-                   part->lines_per_chunk;
+        int64_t chunk = (int64_t) leader.scanline_y;
+        chunk -= (int64_t) part->data_window.min.y;
+        chunk /= part->lines_per_chunk;
+        if (chunk < 0 || chunk > INT32_MAX)
+            return ctxt->print_error (
+                ctxt,
+                EXR_ERR_BAD_CHUNK_LEADER,
+                "Invalid chunk index: %" PRId64
+                " reading scanline %d (datawindow min %d) with lines per chunk %d",
+                chunk,
+                leader.scanline_y,
+                part->data_window.min.y,
+                part->lines_per_chunk);
+
+        *indexio = (int) chunk;
     }
     else
     {
@@ -527,10 +542,10 @@ extract_chunk_table (
         EXR_CONST_CAST (atomic_uintptr_t*, &(part->chunk_table)));
     if (ctable == NULL)
     {
-        int64_t   nread = 0;
-        uintptr_t eptr = 0, nptr = 0;
-        int       complete = 1;
-        uint64_t  maxoff   = ((uint64_t) -1);
+        int64_t      nread = 0;
+        uintptr_t    eptr = 0, nptr = 0;
+        int          complete = 1;
+        uint64_t     maxoff   = ((uint64_t) -1);
         exr_result_t rv;
 
         if (part->chunk_count <= 0)
@@ -565,7 +580,7 @@ extract_chunk_table (
             // file is incomplete (i.e. crashed during write and didn't
             // get a complete chunk table), so just do them one at a time
             if (ctxt->file_size > 0) maxoff = (uint64_t) ctxt->file_size;
-            for (size_t ci = 0; ci < part->chunk_count; ++ci)
+            for (int ci = 0; ci < part->chunk_count; ++ci)
             {
                 uint64_t cchunk = one_to_native64 (ctable[ci]);
                 if (cchunk < chunkoff || cchunk >= maxoff) complete = 0;
@@ -590,10 +605,7 @@ extract_chunk_table (
                 }
             }
         }
-        else
-        {
-            priv_to_native64 (ctable, part->chunk_count);
-        }
+        else { priv_to_native64 (ctable, part->chunk_count); }
 
         nptr = (uintptr_t) ctable;
         // see if we win or not
@@ -670,21 +682,11 @@ compute_chunk_unpack_size (
         {
             const exr_attr_chlist_entry_t* curc = (chanlist->entries + c);
             uint64_t chansz = ((curc->pixel_type == EXR_PIXEL_HALF) ? 2 : 4);
-            chansz *= ((uint64_t) width);
+
+            chansz *= (uint64_t) width;
             if (curc->x_sampling > 1) chansz /= ((uint64_t) curc->x_sampling);
-            chansz *= ((uint64_t) height);
-            if (curc->y_sampling > 1)
-            {
-                if (height > 1)
-                {
-                    if (curc->y_sampling > height)
-                        chansz /= ((uint64_t) height);
-                    else
-                        chansz /= ((uint64_t) curc->y_sampling);
-                }
-                else if ((y % ((int) curc->y_sampling)) != 0)
-                    chansz = 0;
-            }
+            chansz *=
+                (uint64_t) compute_sampled_lines (height, curc->y_sampling, y);
             unpacksize += chansz;
         }
     }
@@ -761,10 +763,7 @@ exr_read_scanline_chunk_info (
         cinfo->start_y = dw.min.y;
         cinfo->height -= (dw.min.y - miny);
     }
-    else if ((miny + lpc) > dw.max.y)
-    {
-        cinfo->height = (dw.max.y - miny + 1);
-    }
+    else if ((miny + lpc) > dw.max.y) { cinfo->height = (dw.max.y - miny + 1); }
     cinfo->level_x = 0;
     cinfo->level_y = 0;
 
@@ -932,10 +931,11 @@ exr_read_scanline_chunk_info (
                 pctxt,
                 EXR_ERR_BAD_CHUNK_LEADER,
                 "Preparing to read scanline %d (chunk %d), found corrupt leader: packed size %" PRIu64
-                ", file size %" PRId64,
+                ", file offset %" PRIu64 ", size %" PRId64,
                 y,
                 cidx,
-                (uint64_t) data[rdcnt],
+                cinfo->packed_size,
+                cinfo->data_offset,
                 fsize);
         }
     }
@@ -1270,7 +1270,7 @@ exr_read_tile_chunk_info (
         else if (fsize > 0)
         {
             uint64_t finpos = dataoff + (uint64_t) tdata[4];
-            if (finpos > fsize)
+            if (finpos > (uint64_t)fsize)
             {
                 return pctxt->print_error (
                     pctxt,
@@ -1332,12 +1332,12 @@ exr_read_chunk (
         return pctxt->report_error (
             pctxt,
             EXR_ERR_INVALID_ARGUMENT,
-            "mis-matched storage type for chunk block info");
+            "mismatched storage type for chunk block info");
     if (cinfo->compression != (uint8_t) part->comp_type)
         return pctxt->report_error (
             pctxt,
             EXR_ERR_INVALID_ARGUMENT,
-            "mis-matched compression type for chunk block info");
+            "mismatched compression type for chunk block info");
 
     dataoffset = cinfo->data_offset;
     if (pctxt->file_size > 0 && dataoffset > (uint64_t) pctxt->file_size)
@@ -1400,12 +1400,12 @@ exr_read_deep_chunk (
         return pctxt->report_error (
             pctxt,
             EXR_ERR_INVALID_ARGUMENT,
-            "mis-matched storage type for chunk block info");
+            "mismatched storage type for chunk block info");
     if (cinfo->compression != (uint8_t) part->comp_type)
         return pctxt->report_error (
             pctxt,
             EXR_ERR_INVALID_ARGUMENT,
-            "mis-matched compression type for chunk block info");
+            "mismatched compression type for chunk block info");
 
     if (pctxt->file_size > 0 &&
         cinfo->sample_count_data_offset > (uint64_t) pctxt->file_size)
@@ -1641,10 +1641,7 @@ write_scan_chunk (
             /* just in case we look at it again? */
             priv_to_native64 (ctable, part->chunk_count);
         }
-        else
-        {
-            pctxt->last_output_chunk = cidx;
-        }
+        else { pctxt->last_output_chunk = cidx; }
     }
 
     return rv;
@@ -1726,10 +1723,7 @@ exr_write_scanline_chunk_info (
         cinfo->start_y = dw.min.y;
         cinfo->height -= (dw.min.y - miny);
     }
-    else if ((miny + lpc) > dw.max.y)
-    {
-        cinfo->height = (dw.max.y - miny + 1);
-    }
+    else if ((miny + lpc) > dw.max.y) { cinfo->height = (dw.max.y - miny + 1); }
     cinfo->level_x = 0;
     cinfo->level_y = 0;
 
@@ -2028,7 +2022,10 @@ write_tile_chunk (
         ddata[0] = (int64_t) sample_data_size;
         ddata[1] = (int64_t) packed_size;
         ddata[2] = (int64_t) unpacked_size;
-        rv       = pctxt->do_write (
+
+        priv_from_native64 (ddata, 3);
+
+        rv = pctxt->do_write (
             pctxt, ddata, 3 * sizeof (uint64_t), &(pctxt->output_file_offset));
 
         if (rv == EXR_ERR_SUCCESS)
@@ -2064,10 +2061,7 @@ write_tile_chunk (
             /* just in case we look at it again? */
             priv_to_native64 (ctable, part->chunk_count);
         }
-        else
-        {
-            pctxt->last_output_chunk = cidx;
-        }
+        else { pctxt->last_output_chunk = cidx; }
     }
 
     return rv;
