@@ -4,6 +4,7 @@
 */
 
 #include "internal_coding.h"
+#include "internal_util.h"
 
 #include <string.h>
 
@@ -40,16 +41,8 @@ internal_coding_fill_channel_info (
 
         decc->channel_name = curc->name.str;
 
-        if (curc->y_sampling > 1)
-        {
-            if (cinfo->height == 1)
-                decc->height = ((cinfo->start_y % curc->y_sampling) == 0) ? 1
-                                                                          : 0;
-            else
-                decc->height = cinfo->height / curc->y_sampling;
-        }
-        else
-            decc->height = cinfo->height;
+        decc->height = compute_sampled_lines (
+            cinfo->height, curc->y_sampling, cinfo->start_y);
 
         if (curc->x_sampling > 1)
             decc->width = cinfo->width / curc->x_sampling;
@@ -106,16 +99,8 @@ internal_coding_update_channel_info (
 
         ccic->channel_name = curc->name.str;
 
-        if (curc->y_sampling > 1)
-        {
-            if (cinfo->height == 1)
-                ccic->height = ((cinfo->start_y % curc->y_sampling) == 0) ? 1
-                                                                          : 0;
-            else
-                ccic->height = cinfo->height / curc->y_sampling;
-        }
-        else
-            ccic->height = cinfo->height;
+        ccic->height = compute_sampled_lines (
+            cinfo->height, curc->y_sampling, cinfo->start_y);
 
         if (curc->x_sampling > 1)
             ccic->width = cinfo->width / curc->x_sampling;
@@ -151,7 +136,7 @@ internal_encode_free_buffer (
                 encode->free_fn (bufid, curbuf);
             else
             {
-                EXR_PROMOTE_CONST_CONTEXT_AND_PART_OR_ERROR_NO_LOCK (
+                EXR_PROMOTE_CONST_CONTEXT_OR_ERROR_NO_PART_NO_LOCK (
                     encode->context, encode->part_index);
 
                 pctxt->free_fn (curbuf);
@@ -174,7 +159,7 @@ internal_encode_alloc_buffer (
     void* curbuf = *buf;
     if (newsz == 0)
     {
-        EXR_PROMOTE_CONST_CONTEXT_AND_PART_OR_ERROR_NO_LOCK (
+        EXR_PROMOTE_CONST_CONTEXT_OR_ERROR_NO_PART_NO_LOCK (
             encode->context, encode->part_index);
 
         return pctxt->print_error (
@@ -192,7 +177,7 @@ internal_encode_alloc_buffer (
             curbuf = encode->alloc_fn (bufid, newsz);
         else
         {
-            EXR_PROMOTE_CONST_CONTEXT_AND_PART_OR_ERROR_NO_LOCK (
+            EXR_PROMOTE_CONST_CONTEXT_OR_ERROR_NO_PART_NO_LOCK (
                 encode->context, encode->part_index);
 
             curbuf = pctxt->alloc_fn (newsz);
@@ -200,7 +185,7 @@ internal_encode_alloc_buffer (
 
         if (curbuf == NULL)
         {
-            EXR_PROMOTE_CONST_CONTEXT_AND_PART_OR_ERROR_NO_LOCK (
+            EXR_PROMOTE_CONST_CONTEXT_OR_ERROR_NO_PART_NO_LOCK (
                 encode->context, encode->part_index);
 
             return pctxt->print_error (
@@ -233,7 +218,7 @@ internal_decode_free_buffer (
                 decode->free_fn (bufid, curbuf);
             else
             {
-                EXR_PROMOTE_CONST_CONTEXT_AND_PART_OR_ERROR_NO_LOCK (
+                EXR_PROMOTE_CONST_CONTEXT_OR_ERROR_NO_PART_NO_LOCK (
                     decode->context, decode->part_index);
 
                 pctxt->free_fn (curbuf);
@@ -271,7 +256,7 @@ internal_decode_alloc_buffer (
             curbuf = decode->alloc_fn (bufid, newsz);
         else
         {
-            EXR_PROMOTE_CONST_CONTEXT_AND_PART_OR_ERROR_NO_LOCK (
+            EXR_PROMOTE_CONST_CONTEXT_OR_ERROR_NO_PART_NO_LOCK (
                 decode->context, decode->part_index);
 
             curbuf = pctxt->alloc_fn (newsz);
@@ -279,7 +264,7 @@ internal_decode_alloc_buffer (
 
         if (curbuf == NULL)
         {
-            EXR_PROMOTE_CONST_CONTEXT_AND_PART_OR_ERROR_NO_LOCK (
+            EXR_PROMOTE_CONST_CONTEXT_OR_ERROR_NO_PART_NO_LOCK (
                 decode->context, decode->part_index);
 
             return pctxt->print_error (
