@@ -386,49 +386,6 @@ exr_start_inplace_header_update (
 
 /**************************************/
 
-exr_result_t
-exr_start_memory_write (
-    exr_context_t* ctxt, const exr_context_initializer_t* ctxtdata)
-{
-    int                           rv    = EXR_ERR_UNKNOWN;
-    struct _internal_exr_context* ret   = NULL;
-    exr_context_initializer_t     inits = fill_context_data (ctxtdata);
-
-    if (!ctxt)
-    {
-        inits.error_handler_fn (
-            NULL,
-            EXR_ERR_INVALID_ARGUMENT,
-            "Invalid context handle passed to start_read function");
-        return EXR_ERR_INVALID_ARGUMENT;
-    }
-
-    rv = internal_exr_alloc_context (
-        &ret,
-        &inits,
-        EXR_CONTEXT_WRITE,
-        sizeof (struct _internal_exr_filehandle));
-    if (rv == EXR_ERR_SUCCESS)
-    {
-        ret->do_write = &dispatch_write;
-
-        rv = exr_attr_string_create (
-            (exr_context_t) ret, &(ret->filename), "<memory>");
-
-        if (rv == EXR_ERR_SUCCESS)
-        {
-            if (!inits.write_fn) rv = default_init_write_file (ret);
-
-            if (rv != EXR_ERR_SUCCESS) exr_finish ((exr_context_t*) &ret);
-        }
-    }
-
-    *ctxt = (exr_context_t) ret;
-    return rv;
-}
-
-/**************************************/
-
 exr_result_t
 exr_get_file_name (exr_const_context_t ctxt, const char** name)
 {
