@@ -35,6 +35,7 @@ convertToLinear (uint16_t s[16])
 static inline int
 shiftAndRound (int x, int shift)
 {
+    int a, b;
     //
     // Compute
     //
@@ -47,9 +48,9 @@ shiftAndRound (int x, int shift)
     //
 
     x <<= 1;
-    int a = (1 << shift) - 1;
+    a = (1 << shift) - 1;
     shift += 1;
-    int b = (x >> shift) & 1;
+    b = (x >> shift) & 1;
     return (x + a + b) >> shift;
 }
 
@@ -108,6 +109,8 @@ pack (const uint16_t s[16], uint8_t b[14], int flatfields, int exactmax)
     uint16_t tMax;
     int      shift = -1;
 
+    const int bias = 0x20;
+
     for (int i = 0; i < 16; ++i)
     {
         if ((s[i] & 0x7c00) == 0x7c00)
@@ -126,13 +129,11 @@ pack (const uint16_t s[16], uint8_t b[14], int flatfields, int exactmax)
     //
     // Compute a set of running differences, r[0] ... r[14]:
     // Find a shift value such that after rounding off the
-    // rightmost bits and shifting all differenes are between
+    // rightmost bits and shifting all differences are between
     // -32 and +31.  Then bias the differences so that they
     // end up between 0 and 63.
     //
 
-    const int bias = 0x20;
-
     do
     {
         shift += 1;
@@ -234,10 +235,11 @@ pack (const uint16_t s[16], uint8_t b[14], int flatfields, int exactmax)
 static inline void
 unpack14 (const uint8_t b[14], uint16_t s[16])
 {
+    uint16_t shift, bias;
     s[0] = ((uint16_t) (b[0] << 8)) | ((uint16_t) b[1]);
 
-    uint16_t shift = (b[2] >> 2);
-    uint16_t bias  = (uint16_t) (0x20u << shift);
+    shift = (b[2] >> 2);
+    bias  = (uint16_t) (0x20u << shift);
 
     s[4] =
         (uint16_t) ((uint32_t) s[0] + (uint32_t) ((((uint32_t) (b[2] << 4) | (uint32_t) (b[3] >> 4)) & 0x3fu) << shift) - bias);
@@ -345,10 +347,7 @@ compress_b44_impl (exr_encode_pipeline_t* encode, int flat_field)
                 }
                 tmp += ((uint64_t) (y / curc->y_samples)) * bpl;
             }
-            else
-            {
-                tmp += ((uint64_t) y) * bpl;
-            }
+            else { tmp += ((uint64_t) y) * bpl; }
 
             memcpy (tmp, packed, bpl);
             if (curc->data_type == EXR_PIXEL_HALF) priv_to_native16 (tmp, nx);
@@ -390,12 +389,14 @@ compress_b44_impl (exr_encode_pipeline_t* encode, int flat_field)
             // by 4, then pad the data by repeating the
             // rightmost column and the bottom row.
             //
+            uint16_t* row0, * row1, * row2, * row3;
 
-            uint16_t* row0 = (uint16_t*) scratch;
+            row0 = (uint16_t*) scratch;
             row0 += y * nx;
-            uint16_t* row1 = row0 + nx;
-            uint16_t* row2 = row1 + nx;
-            uint16_t* row3 = row2 + nx;
+
+            row1 = row0 + nx;
+            row2 = row1 + nx;
+            row3 = row2 + nx;
 
             if (y + 3 >= ny)
             {
@@ -608,6 +609,36 @@ uncompress_b44_impl (
     return EXR_ERR_SUCCESS;
 }
 
+/**************************************/
+
+static uint64_t
+compute_scratch_buffer_size (
+    exr_decode_pipeline_t* decode, uint64_t uncompressed_size)
+{
+    const exr_coding_channel_info_t* curc;
+    int                              nx, ny;
+    uint64_t                         ret  = uncompressed_size;
+    uint64_t                         comp = 0;
+
+    for (int c = 0; c < decode->channel_count; ++c)
+    {
+        curc = decode->channels + c;
+
+        nx = curc->width;
+        ny = curc->height;
+
+        if (nx % 4) nx += 4 - nx % 4;
+        if (ny % 4) ny += 4 - ny % 4;
+
+        comp += (uint64_t) (ny) * (uint64_t) (nx) *
+                (uint64_t) (curc->bytes_per_element);
+    }
+    if (comp > ret) ret = comp;
+    return ret;
+}
+
+/**************************************/
+
 exr_result_t
 internal_exr_undo_b44 (
     exr_decode_pipeline_t* decode,
@@ -622,7 +653,7 @@ internal_exr_undo_b44 (
         EXR_TRANSCODE_BUFFER_SCRATCH1,
         &(decode->scratch_buffer_1),
         &(decode->scratch_alloc_size_1),
-        uncompressed_size);
+        compute_scratch_buffer_size (decode, uncompressed_size));
     if (rv != EXR_ERR_SUCCESS) return rv;
 
     return uncompress_b44_impl (
@@ -647,7 +678,7 @@ internal_exr_undo_b44a (
         EXR_TRANSCODE_BUFFER_SCRATCH1,
         &(decode->scratch_buffer_1),
         &(decode->scratch_alloc_size_1),
-        uncompressed_size);
+        compute_scratch_buffer_size (decode, uncompressed_size));
     if (rv != EXR_ERR_SUCCESS) return rv;
 
     return uncompress_b44_impl (
