@@ -177,7 +177,7 @@ wav_2D_encode (uint16_t* in, int nx, int ox, int ny, int oy, uint16_t mx)
     int p2  = 2; // == 1 << (level+1)
 
     //
-    // Hierachical loop on smaller dimension n
+    // Hierarchical loop on smaller dimension n
     //
 
     while (p2 <= n)
@@ -464,10 +464,7 @@ internal_exr_apply_piz (exr_encode_pipeline_t* encode)
                 if ((cury % curc->y_samples) != 0) continue;
                 tmp += ((uint64_t) (y / curc->y_samples)) * bpl;
             }
-            else
-            {
-                tmp += ((uint64_t) y) * bpl;
-            }
+            else { tmp += ((uint64_t) y) * bpl; }
 
             memcpy (tmp, packed, bpl);
             priv_to_native16 (tmp, nx * (curc->bytes_per_element / 2));
@@ -524,16 +521,26 @@ internal_exr_apply_piz (exr_encode_pipeline_t* encode)
         ndata,
         hufspare,
         hufSpareBytes);
-    if (rv != EXR_ERR_SUCCESS) return rv;
-    nOut += nBytes;
-    if (nOut < packedbytes)
+    if (rv != EXR_ERR_SUCCESS)
     {
-        unaligned_store32 (lengthptr, (uint32_t) nBytes);
+        if (rv == EXR_ERR_ARGUMENT_OUT_OF_RANGE)
+        {
+            memcpy (encode->compressed_buffer, encode->packed_buffer, packedbytes);
+            nOut = packedbytes;
+        }
     }
     else
     {
-        memcpy (encode->compressed_buffer, encode->packed_buffer, packedbytes);
-        nOut = packedbytes;
+        nOut += nBytes;
+        if (nOut < packedbytes)
+        {
+            unaligned_store32 (lengthptr, (uint32_t) nBytes);
+        }
+        else
+        {
+            memcpy (encode->compressed_buffer, encode->packed_buffer, packedbytes);
+            nOut = packedbytes;
+        }
     }
     encode->compressed_bytes = nOut;
     return EXR_ERR_SUCCESS;
@@ -625,7 +632,13 @@ internal_exr_undo_piz (
 
     wavbuf = decode->scratch_buffer_1;
     rv     = internal_huf_decompress (
-        packed + nBytes, hufbytes, wavbuf, outsz / 2, hufspare, hufSpareBytes);
+        decode,
+        packed + nBytes,
+        hufbytes,
+        wavbuf,
+        outsz / 2,
+        hufspare,
+        hufSpareBytes);
     if (rv != EXR_ERR_SUCCESS) return rv;
 
     //
