@@ -7,7 +7,6 @@
 #include "internal_attr.h"
 #include "internal_constants.h"
 #include "internal_memory.h"
-#include "backward_compatibility.h"
 
 #include <IlmThreadConfig.h>
 
@@ -32,7 +31,7 @@ default_error_handler (
 {
     const struct _internal_exr_context* pctxt = EXR_CCTXT (ctxt);
 
-#ifdef ILMBASE_THREADING_ENABLED
+#ifdef ILMTHREAD_THREADING_ENABLED
 #    ifdef _WIN32
     static CRITICAL_SECTION sMutex;
     volatile static long    initialized = 0;
@@ -44,7 +43,7 @@ default_error_handler (
 #    endif
 #endif
 
-#ifdef ILMBASE_THREADING_ENABLED
+#ifdef ILMTHREAD_THREADING_ENABLED
 #    ifdef _WIN32
     EnterCriticalSection (&sMutex);
 #    else
@@ -72,7 +71,7 @@ default_error_handler (
         fprintf (stderr, "<ERROR>: %s\n", msg);
     fflush (stderr);
 
-#ifdef ILMBASE_THREADING_ENABLED
+#ifdef ILMTHREAD_THREADING_ENABLED
 #    ifdef _WIN32
     LeaveCriticalSection (&sMutex);
 #    else
@@ -337,10 +336,10 @@ internal_exr_alloc_context (
     void*                         memptr;
     exr_result_t                  rv;
     struct _internal_exr_context* ret;
-    *out = NULL;
     int    gmaxw, gmaxh;
     size_t extra_data;
 
+    *out = NULL;
     if (initializers->read_fn || initializers->write_fn)
         extra_data = 0;
     else
@@ -418,6 +417,7 @@ internal_exr_alloc_context (
         ret->disable_chunk_reconstruct =
             (initializers->flags &
              EXR_CONTEXT_FLAG_DISABLE_CHUNK_RECONSTRUCTION);
+        ret->legacy_header = (initializers->flags & EXR_CONTEXT_FLAG_WRITE_LEGACY_HEADER);
 
         ret->file_size       = -1;
         ret->max_name_length = EXR_SHORTNAME_MAXLEN;
