@@ -5,18 +5,13 @@
 
 #include "internal_coding.h"
 #include "internal_xdr.h"
+#include "internal_cpuid.h"
 
 #include "openexr_attr.h"
 
 #include <string.h>
 #include <stdbool.h>
 
-#if defined(__x86_64__) || defined(_M_X64)
-#    ifndef _WIN32
-#        include <cpuid.h>
-#    endif
-#endif
-
 /**************************************/
 
 #ifndef __F16C__
@@ -37,7 +32,7 @@ half_to_float8 (float* out, const uint16_t* src)
 }
 #endif
 
-#if (defined(__x86_64__) || defined(_M_X64)) &&                                \
+#if (defined(__x86_64__) || defined(_M_X64)) && defined(__AVX__) &&                              \
     (defined(__F16C__) || defined(__GNUC__) || defined(__clang__))
 
 #    if defined(__F16C__)
@@ -141,33 +136,15 @@ half_to_float_buffer_impl (float* out, const uint16_t* in, int w)
 static void (*half_to_float_buffer) (float*, const uint16_t*, int) =
     &half_to_float_buffer_impl;
 
-static void
-choose_half_to_float_impl ()
+static inline void
+choose_half_to_float_impl (void)
 {
-    // regs[2] in the extended block is ECX, where f16c indicator lives
-#        ifdef _WIN32
-    int regs[4];
-
-    __cpuid (regs, 0);
-    if (regs[0] >= 1) { __cpuidex (regs, 1, 0); }
-    else regs[2] = 0;
-#        else
-    unsigned int regs[4];
-    __get_cpuid (0, &regs[0], &regs[1], &regs[2], &regs[3]);
-    if (regs[0] >= 1)
-    {
-        __get_cpuid (1, &regs[0], &regs[1], &regs[2], &regs[3]);
-    }
-    else
-        regs[2] = 0;
-#        endif
-    /* F16C is indicated by bit 29 */
-    if (regs[2] & (1 << 29)) half_to_float_buffer = &half_to_float_buffer_f16c;
+    if (has_native_half ()) half_to_float_buffer = &half_to_float_buffer_f16c;
 }
 #    else
 /* when we explicitly compile against f16, force it in */
-static void
-choose_half_to_float_impl ()
+static inline void
+choose_half_to_float_impl (void)
 {}
 
 #    endif /* F16C */
@@ -221,7 +198,7 @@ half_to_float_buffer (float* out, const uint16_t* in, int w)
 }
 
 static void
-choose_half_to_float_impl ()
+choose_half_to_float_impl (void)
 {}
 
 #endif
@@ -1228,9 +1205,10 @@ generic_unpack_deep_pointers (exr_decode_pipeline_t* decode)
                 if (outpix)
                 {
                     uint8_t* cdata = outpix;
+
                     UNPACK_SAMPLES (samps)
                 }
-                srcbuffer += bpc * samps;
+                srcbuffer += ((size_t) bpc) * ((size_t) samps);
             }
         }
         sampbuffer += w;
@@ -1274,12 +1252,14 @@ generic_unpack_deep (exr_decode_pipeline_t* decode)
                 }
                 else
                     prevsamps = sampbuffer[w - 1];
+
                 srcbuffer += ((size_t) bpc) * ((size_t) prevsamps);
 
                 if (incr_tot) totsamps += (size_t) prevsamps;
 
                 continue;
             }
+
             cdata += totsamps * ((size_t) ubpc);
 
             for (int x = 0; x < w; ++x)
@@ -1295,7 +1275,7 @@ generic_unpack_deep (exr_decode_pipeline_t* decode)
 
                 UNPACK_SAMPLES (samps)
 
-                srcbuffer += bpc * samps;
+                srcbuffer += ((size_t) bpc) * ((size_t) samps);
                 if (incr_tot) totsamps += (size_t) samps;
             }
         }
@@ -1333,7 +1313,7 @@ internal_exr_match_decode (
 
     if (isdeep)
     {
-        if ((decode->decode_flags & EXR_DECODE_SAMPLE_COUNTS_AS_INDIVIDUAL))
+        if ((decode->decode_flags & EXR_DECODE_NON_IMAGE_DATA_AS_POINTERS))
             return &generic_unpack_deep_pointers;
         return &generic_unpack_deep;
     }
