@@ -50,7 +50,7 @@ validate_req_attr (
     {
         if (adddefault)
         {
-            exr_attr_box2i_t defdw = { 0, 0, 63, 63 };
+            exr_attr_box2i_t defdw = {{.x = 0, .y = 0}, {.x = 63, .y = 63}};
             rv                     = exr_attr_list_add_static_name (
                 (exr_context_t) f,
                 &(curpart->attributes),
@@ -78,7 +78,7 @@ validate_req_attr (
     {
         if (adddefault)
         {
-            exr_attr_box2i_t defdw = { 0, 0, 63, 63 };
+            exr_attr_box2i_t defdw = {{.x = 0, .y = 0}, {.x = 63, .y = 63}};
             rv                     = exr_attr_list_add_static_name (
                 (exr_context_t) f,
                 &(curpart->attributes),
@@ -151,7 +151,7 @@ validate_req_attr (
     {
         if (adddefault)
         {
-            exr_attr_v2f_t defswc = { 0.f, 0.f };
+            exr_attr_v2f_t defswc = {.x = 0.f, .y = 0.f};
             rv                    = exr_attr_list_add_static_name (
                 (exr_context_t) f,
                 &(curpart->attributes),
@@ -420,7 +420,7 @@ validate_part_type (
         if (f->is_singlepart_tiled &&
             curpart->storage_mode != EXR_STORAGE_TILED)
         {
-            // mis-match between type attr and file flag. c++ believed the
+            // mismatch between type attr and file flag. c++ believed the
             // flag first and foremost
             curpart->storage_mode = EXR_STORAGE_TILED;
 
