@@ -489,7 +489,7 @@ make_pattern(data<DATA> & bob,int pattern_number)
 
 template<class T>
 void
-write_file(const char * filename, const data<T> & master, int number_of_parts)
+write_file(const char * filename, const data<T> & main, int number_of_parts)
 {
     vector<Header> headers(number_of_parts);
     
@@ -518,12 +518,12 @@ write_file(const char * filename, const data<T> & master, int number_of_parts)
     
     if(number_of_parts>1)
     {
-      master.frak(sub_parts);
+      main.frak(sub_parts);
     }  
     
     if(number_of_parts==1)
     {
-        master.setHeader(headers[0]);
+        main.setHeader(headers[0]);
     }else{
         
         for(int i=0;i<number_of_parts;i++)
@@ -538,7 +538,7 @@ write_file(const char * filename, const data<T> & master, int number_of_parts)
         DeepScanLineOutputPart p(f,i);
         if(number_of_parts==1)
         {
-            master.writeData(p);
+            main.writeData(p);
         }else{
             sub_parts[i].writeData(p);
         }
@@ -555,9 +555,9 @@ test_parts (int pattern_number,
 {
      std::string fn = tempDir + "imf_test_composite_deep_scanline_source.exr";
 
-     data<T> master;
-     make_pattern (master, pattern_number);
-     write_file (fn.c_str(), master,number_of_parts);
+     data<T> main;
+     make_pattern (main, pattern_number);
+     write_file (fn.c_str(), main,number_of_parts);
      
 
      {
@@ -576,7 +576,7 @@ test_parts (int pattern_number,
         }
      
      
-        master.setUpFrameBuffer(data,testbuf,comp.dataWindow(),load_depths);
+        main.setUpFrameBuffer(data,testbuf,comp.dataWindow(),load_depths);
      
         comp.setFrameBuffer(testbuf);
      
@@ -598,7 +598,7 @@ test_parts (int pattern_number,
            }
          }
         
-         master.checkValues(data,comp.dataWindow(),load_depths);
+         main.checkValues(data,comp.dataWindow(),load_depths);
   
          for(int i=0;i<number_of_parts;i++)
          {
@@ -612,7 +612,7 @@ test_parts (int pattern_number,
          vector<T> data;
          FrameBuffer testbuf;
          const Box2i & dataWindow = file.header().dataWindow();
-         master.setUpFrameBuffer(data,testbuf,dataWindow,load_depths);
+         main.setUpFrameBuffer(data,testbuf,dataWindow,load_depths);
          file.setFrameBuffer(testbuf);
          if(entire_buffer)
          {
@@ -629,7 +629,7 @@ test_parts (int pattern_number,
              }
          }
          
-         master.checkValues (data, dataWindow, load_depths);
+         main.checkValues (data, dataWindow, load_depths);
          
      }
      remove (fn.c_str());
