@@ -365,7 +365,7 @@ generateRandomHeaders (int partCount, vector<Header>& headers)
                  << " tile order =" << header.lineOrder()
                  << " levelMode = " << levelModes[i] << endl << flush;
         }
-        // future types MUST have a chunkCount attribute - ommitting causes the library
+        // future types MUST have a chunkCount attribute - omitting causes the library
         // to raise an exception (can't compute chunkOffsetTable) and prevents us from reading
         // the rest of the image
         header.setChunkCount(getChunkOffsetTableSize(header));
@@ -1314,7 +1314,8 @@ testWriteRead (int partNumber)
             }
             catch(std::exception & e)
             {
-                cout << "recieved exception (" << e.what() << ") as expected\n";
+                cout << "received exception (" << e.what ()
+                     << ") as expected\n";
                 caught = true;
                 // that's what we thought would happen
             }
@@ -1333,7 +1334,7 @@ testWriteRead (int partNumber)
         }
         catch(std::exception & e)
         {
-            cout << "recieved exception (" << e.what() << ") as expected\n";
+            cout << "received exception (" << e.what () << ") as expected\n";
             caught = true;
             // that's what we thought would happen
         }
