@@ -247,16 +247,21 @@ bool compare(const FrameBuffer& asRead,
 // allocate readingBuffer or writingBuffer, setting up a framebuffer to point to the right thing
 //
 ChannelList
-setupBuffer (const Header& hdr,       // header to grab datawindow from
-             const char * const *channels, // NULL terminated list of channels to write
-             const char * const *passivechannels, // NULL terminated list of channels to write
-             const PixelType* pt,     // type of each channel, or NULL for all HALF
-             FrameBuffer& buf,        // buffer to fill with pointers to channel
-             FrameBuffer& prereadbuf, // channels which aren't being read - indexes into the preread buffer
-             FrameBuffer& postreadbuf, // channels which aren't being read - indexes into the postread buffer
-             int banks,                    // number of banks - channels within each bank are interleaved, banks are scanline interleaved
-             bool writing                  // true if should allocate
-            )
+setupBuffer (
+    const Header&      hdr,      // header to grab datawindow from
+    const char* const* channels, // NULL terminated list of channels to write
+    const char* const*
+        passivechannels,  // NULL terminated list of channels to write
+    const PixelType* pt,  // type of each channel, or NULL for all HALF
+    FrameBuffer&     buf, // buffer to fill with pointers to channel
+    FrameBuffer&
+        prereadbuf, // channels which aren't being read - indexes into the preread buffer
+    FrameBuffer&
+         postreadbuf, // channels which aren't being read - indexes into the postread buffer
+    int  banks, // number of banks - channels within each bank are interleaved, banks are scanline interleaved
+    bool writing, // true if should allocate
+    bool allowNonfinite // true if the buffer is allowed to create infinity or NaN values
+)
 {
     Box2i dw = hdr.dataWindow();
 
@@ -276,10 +281,11 @@ setupBuffer (const Header& hdr,       // header to grab datawindow from
         {
             switch (pt[activechans])
             {
-                case IMF::HALF : bytes_per_pixel+=2;break;
-                case IMF::FLOAT : case IMF::UINT : 
-                    // some architectures (e.g arm7 cannot write 32 bit values
-                    // to addresses which aren't aligned to 32 bit addresses)
+                case IMF::HALF: bytes_per_pixel += 2; break;
+                case IMF::FLOAT:
+                case IMF::UINT:
+                    // some architectures (e.g arm7) cannot write 32 bit values
+                    // to addresses which aren't aligned to 32 bit addresses
                     // so bump to next multiple of four
                     bytes_per_pixel =  alignToFour(bytes_per_pixel);
                     bytes_per_pixel+=4;
@@ -345,41 +351,46 @@ setupBuffer (const Header& hdr,       // header to grab datawindow from
         readingBuffer.resize(size);
     }
    
-     const char * write_ptr = writing ? &writingBuffer[0] : &readingBuffer[0];
-     // fill with random halfs, casting to floats for float channels
-     int chan=0;
-     for (int i=0;i<samples;i++)
-     {
-         unsigned short int values = random_int(std::numeric_limits<unsigned short>::max());
-         half v;
-         v.setBits(values);
-         if (pt==NULL || pt[chan]==IMF::HALF)
-         {
-             *(half*)write_ptr = half(v);
-             write_ptr+=2;
-         }
-         else
-         {
-             write_ptr = alignToFour(write_ptr);
-             *(float*)write_ptr = float(v);
-             
-             write_ptr+=4;
-         }
-         chan++;
-         if (chan==chans)
-         {
-             chan=0;
-         }
-     
-     }
+    const char* write_ptr = writing ? &writingBuffer[0] : &readingBuffer[0];
+    // fill with random halfs, casting to floats for float channels
+    int chan = 0;
+    for (int i = 0; i < samples; i++)
+    {
+        half v;
+        // generate a random finite half
+        // if the value is to be cast to a float, ensure the value is not infinity
+        // or NaN, since the value is not guaranteed to round trip from half to float
+        // and back again with bit-identical precision
+        do
+        {
+            unsigned short int values =
+                random_int (std::numeric_limits<unsigned short>::max ());
+            v.setBits (values);
+        } while(!( (v-v)==0 || allowNonfinite ));
 
-     if (!writing)
-     {
-         //take a copy of the buffer as it was before being read
-         preReadBuffer = readingBuffer;
-     }
-     
-    char* offset=NULL;
+        if (pt == NULL || pt[chan] == IMF::HALF)
+        {
+            *(half*) write_ptr = half (v);
+            write_ptr += 2;
+        }
+        else
+        {
+            write_ptr           = alignToFour (write_ptr);
+            *(float*) write_ptr = float (v);
+
+            write_ptr += 4;
+        }
+        chan++;
+        if (chan == chans) { chan = 0; }
+    }
+
+    if (!writing)
+    {
+        //take a copy of the buffer as it was before being read
+        preReadBuffer = readingBuffer;
+    }
+
+    char* offset = NULL;
 
     ChannelList chanlist;
 
@@ -464,9 +475,8 @@ setupBuffer (const Header& hdr,       // header to grab datawindow from
     return chanlist;
 }
 
-
-
-Box2i writefile(Schema & scheme,FrameBuffer& buf,bool tiny)
+Box2i
+writefile (Schema& scheme, FrameBuffer& buf, bool tiny , bool allowNonfinite)
 {
     const int height = 128;
     const int width  = 128;
@@ -504,7 +514,8 @@ Box2i writefile(Schema & scheme,FrameBuffer& buf,bool tiny)
                                   dummy1,
                                   dummy2,
                                   scheme._banks,
-                                  true);
+                                  true,
+                                  allowNonfinite);
     
     if (scheme._views)
     {
@@ -516,29 +527,36 @@ Box2i writefile(Schema & scheme,FrameBuffer& buf,bool tiny)
     f.setFrameBuffer(buf);
     f.writePixels(hdr.dataWindow().max.y-hdr.dataWindow().min.y+1);
 
-    return hdr.dataWindow();
+    return hdr.dataWindow ();
 }
 
 bool
-readfile (Schema scheme,
-          FrameBuffer & buf,      ///< list of channels to read: index to readingBuffer
-          FrameBuffer & preread,  ///< list of channels to skip: index to preReadBuffer
-          FrameBuffer & postread) ///< list of channels to skip: index to readingBuffer)
+readfile (
+    Schema       scheme,
+    FrameBuffer& buf,     ///< list of channels to read: index to readingBuffer
+    FrameBuffer& preread, ///< list of channels to skip: index to preReadBuffer
+    FrameBuffer&
+        postread, ///< list of channels to skip: index to readingBuffer)
+    bool allowNonfinite)
 {
-    InputFile infile (filename.c_str());
-    setupBuffer(infile.header(),
-                scheme._active,
-                scheme._passive,
-                scheme._types,
-                buf,
-                preread,
-                postread,
-                scheme._banks,false);
-    infile.setFrameBuffer(buf);
-    
-    cout.flush();
-    infile.readPixels (infile.header().dataWindow().min.y,
-                       infile.header().dataWindow().max.y);
+    InputFile infile (filename.c_str ());
+    setupBuffer (
+        infile.header (),
+        scheme._active,
+        scheme._passive,
+        scheme._types,
+        buf,
+        preread,
+        postread,
+        scheme._banks,
+        false,
+        allowNonfinite);
+    infile.setFrameBuffer (buf);
+
+    cout.flush ();
+    infile.readPixels (
+        infile.header ().dataWindow ().min.y,
+        infile.header ().dataWindow ().max.y);
 
     return infile.isOptimizationEnabled();
 }
@@ -551,18 +569,18 @@ test (Schema writeScheme, Schema readScheme, bool nonfatal, bool tiny)
     cout << left << setw(53) << q.str();
 
     FrameBuffer writeFrameBuf;
-    Box2i dw = writefile(writeScheme,writeFrameBuf,tiny);
+    // only allow NaN and infinity values if file is read and written as half float
+    // (otherwise casting between half and float may cause different bit patterns)
+    bool allowNonfinite = (writeScheme._types == nullptr && readScheme._types==nullptr);
+    Box2i       dw = writefile (writeScheme, writeFrameBuf, tiny , allowNonfinite);
     FrameBuffer readFrameBuf;
     FrameBuffer preReadFrameBuf;
     FrameBuffer postReadFrameBuf;
-    cout.flush();
-    bool opt = readfile (readScheme,
-                         readFrameBuf,
-                         preReadFrameBuf,
-                         postReadFrameBuf);
-    if (compare(readFrameBuf, writeFrameBuf, dw, nonfatal) &&
-        compare(preReadFrameBuf, postReadFrameBuf, dw, nonfatal)
-    )
+    cout.flush ();
+    bool opt =
+        readfile (readScheme, readFrameBuf, preReadFrameBuf, postReadFrameBuf,allowNonfinite);
+    if (compare (readFrameBuf, writeFrameBuf, dw, nonfatal) &&
+        compare (preReadFrameBuf, postReadFrameBuf, dw, nonfatal))
     {
         cout <<  " OK ";
         if (opt)
